#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# IDE: PyCharm 2017.2.4
# Author: Dajiang Ren<rendajiang@gagogroup.com>
# Created on 2018-06-13


from .storage import StorageService, Bucket, Object, ObjectNotExistsError, ObjectExistsError, ACLError
